/// <reference types="node" />
import { Socket } from 'net';
import { Message } from './Message';
import { Destroyable } from '../utils/Destroyable';
import { SocketKeepAlive } from './SocketKeepAlive';
/**
 * A enum of all the net socket events.
 */
export declare const enum NetSocketEvents {
    ERROR = "error",
    DATA = "data",
    CLOSE = "close"
}
/**
 * TODO: More private.
 *
 * @author Bram Willem van der Kuip <vdkuipb@gmail.com>
 */
export declare class Client implements Destroyable {
    /**
     * The unique id of this client instance.
     */
    id: string;
    /**
     * The socket that is listened to an writen data to for this client.
     */
    socket: Socket;
    /**
     * Object that handles the ping/pong events.
     */
    keepAlive: SocketKeepAlive;
    /**
     * Bound methods that are saved so they can be removed if this client is disconnected.
     *
     */
    boundOnSocketData: (data: object) => any;
    boundOnSocketClose: () => any;
    boundOnSocketError: () => any;
    /**
     * Creates an instance of Client.
     * @param {Socket} socket The socket of this client.
     * @param {string} id The id of this client.
     */
    constructor(socket: Socket, id: string);
    /**
     * Sends message to the socket of this client.
     * @param {Message} The message to send.
     */
    send(message: Message): void;
    /**
     * Destroys this instance of Client.
     */
    destroyInstance(): void;
}
