"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Message_1 = require("./Message");
var SocketKeepAlive_1 = require("./SocketKeepAlive");
/**
 *
 */
var SOCKET_WRITE_ENCODING = 'binary';
/**
 * TODO: More private.
 *
 * @author Bram Willem van der Kuip <vdkuipb@gmail.com>
 */
var Client = /** @class */ (function () {
    /**
     * Creates an instance of Client.
     * @param {Socket} socket The socket of this client.
     * @param {string} id The id of this client.
     */
    function Client(socket, id) {
        this.socket = socket;
        this.id = id;
        this.keepAlive = new SocketKeepAlive_1.SocketKeepAlive(socket);
    }
    /**
     * Sends message to the socket of this client.
     * @param {Message} The message to send.
     */
    Client.prototype.send = function (message) {
        var data = Message_1.MessageFormatter.encrypt(message);
        this.socket.write(data, SOCKET_WRITE_ENCODING);
    };
    /**
     * Destroys this instance of Client.
     */
    Client.prototype.destroyInstance = function () {
        this.keepAlive.destroyInstance();
        delete this.keepAlive;
        this.socket.removeListener("close" /* CLOSE */, this.boundOnSocketClose);
        this.socket.removeListener("data" /* DATA */, this.boundOnSocketData);
        this.socket.destroy();
        delete this.id;
        delete this.socket;
        delete this.boundOnSocketClose;
        delete this.boundOnSocketData;
    };
    return Client;
}());
exports.Client = Client;
