"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var WebSocketFrame_1 = require("../utils/WebSocketFrame");
var typeString = 'string';
var typeNumber = 'number';
var typeObject = 'object';
/**
 *
 */
var closeMessage = {
    event: "close" /* CLOSE */,
    data: {},
};
/**
 *
 */
var pongMessage = {
    event: "pong" /* PONG */,
    data: {},
};
/**
 *
 */
var INVALID_STRING_PARSE = 'invalid_string_parse';
/**
 *
 */
var INVALID_MESSAGE_OBJECT = 'invalid_message_object';
/**
 *
 */
function invalidDataMessage(type) {
    return {
        event: "invalid_message" /* INVALID_MESSAGE */,
        data: { type: type },
    };
}
/**
 *
 */
var opcodePong = 0x0A;
/**
 *
 */
var opcodeClose = 0x08;
/**
 * TODO: rename.
 *
 * @author Bram Willem van der Kuip <vdkuipb@gmail.com>
 */
var MessageFormatter = /** @class */ (function () {
    function MessageFormatter() {
    }
    /**
     *
     * @param {Message} message
     * @returns {string}
     */
    MessageFormatter.encrypt = function (message) {
        return WebSocketFrame_1.WebSocketFrame.to(message).toBuffer();
    };
    /**
     *
     * @param {object} data
     * @return {Message}
     */
    MessageFormatter.decrypt = function (data) {
        var frame = WebSocketFrame_1.WebSocketFrame.from(data);
        if (frame.opcode === opcodePong) {
            return pongMessage;
        }
        if (frame.opcode === opcodeClose) {
            return closeMessage;
        }
        if (!frame.binaryPayload || frame.binaryPayload.length === 0) {
            return;
        }
        var dataString = frame.binaryPayload.toString();
        var dataObject;
        try {
            dataObject = JSON.parse(dataString);
        }
        catch (error) {
            return invalidDataMessage(INVALID_STRING_PARSE);
        }
        if (!this.isMessage(dataObject)) {
            return invalidDataMessage(INVALID_MESSAGE_OBJECT);
        }
        return dataObject;
    };
    /**
     * Checks if object implements the Message interface.
     * @param {*} data The object the check if is a message.
     * @returns {boolean} The flag if it is true of not.
     */
    MessageFormatter.isMessage = function (data) {
        return data &&
            data.event &&
            (typeof (data.event) === typeString ||
                typeof (data.event) === typeNumber) &&
            data.data !== undefined;
    };
    return MessageFormatter;
}());
exports.MessageFormatter = MessageFormatter;
