import { Client } from './Client';
import { Message } from './Message';
import { Destroyable } from '../utils/Destroyable';
/**
 * An enum of all the net server events.
 */
export declare const enum NetServerEvents {
    ERROR = "error",
    CONNECTION = "connection"
}
/**
 *
 */
export declare const enum BaseMessageEvents {
    CLOSE = "close",
    INVALID_MESSAGE = "invalid_message",
    PONG = "pong",
    PING = "ping"
}
/**
 * @author Bram Willem van der Kuip <vdkuipb@gmail.com>
 */
export declare abstract class Server implements Destroyable {
    /**
     * The ip of this server.
     */
    private ip;
    /**
     * The port this server listens to.
     */
    private port;
    /**
     * An array of al the connected clients.
     */
    private clients;
    /**
     * The net.server isntance.
     */
    private server;
    /**
     * Part of the handshake string that we create beforehand with the ip and port.
     */
    private handshakeAfterAccept;
    /**
     * Bound methods that are saved so they can be removed if this server is destroyed.
     */
    private boundOnError;
    private boundOnConnection;
    /**
     * Creates an instance of Server.
     * @param {string} [ip='127.0.0.1']
     * @param {number} [port=3000]
     */
    constructor(ip?: string, port?: number);
    /**
     * TODO: Add the clientError event.
     * Starts the instance of this Server.
     */
    start(): void;
    /**
     *
     * @param {Error} error
     */
    private onError;
    /**
     *
     * @param {Error} error
     */
    private onClientError;
    /**
     * Called the first time a socket is connected to the server.
     * Lets the server listen once for the data event to handle the handshake.
     * @param {Socket} socket
     */
    private onConnection;
    /**
     *
     * @param {Client} client
     */
    protected abstract onClientConnection(client: Client): void;
    /**
     * TODO: If returns false close socket and remove it.
     * Handles the handshake of the socket and the server.
     * Return if the handshake went succesfully
     * @param {Socket} socket
     * @param {object} handshake
     * @returns {boolean}
     */
    private acceptHandshake;
    /**
     * TODO: Maybe find another solution for this.
     * Handles the client events and is called if the handshake was succesfull.
     * @param {Socket} socket
     */
    private onAcceptedHandshake;
    /**
     * TODO: Handle incorrect message format.
     * Called when a socket sends data to the server.
     * @param {Client} client
     * @param {object} data
     */
    private onData;
    /**
     *
     * @param {Client} client
     * @param {Message} message
     */
    protected abstract onClientData(client: Client, message: Message): void;
    /**
     * TODO: Check the bounds and removes.
     * Called when a socket closes.
     * @param {Client} client
     */
    private onClose;
    /**
     *
     * @param {Client} client
     */
    protected abstract onClientClose(client: Client): void;
    /**
     * Sends a given message to the given client.
     * @param {Client} client
     * @param {Message} message
     */
    send(client: Client, message: Message): void;
    /**
     * TODO: Now send message is doing the message to buffer for every client.
     * Sends a given message to all clients connected.
     * @param {Message} message
     */
    broadcast(message: Message): void;
    /**
     * Destroys this instance of Server.
     */
    destroyInstance(): void;
}
