"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tempWebSocketFrame = require('../vendor/websocket/lib/WebSocketFrame.js');
/**
 * @author Bram Willem van der Kuip <vdkuipb@gmail.com>
 */
var SocketKeepAlive = /** @class */ (function () {
    /**
     * Creates an instance of SocketKeepAlive.
     */
    function SocketKeepAlive(socket) {
        this.socket = socket;
        this.keepAlive = null;
        this.latency = 0;
        this.lastPing = 0;
        this.boundCheckConnection = this.check.bind(this);
        this.boundTimedOut = this.timedOut.bind(this);
        this.reset();
    }
    /**
     * TODO: Move variable to config/constant file.
     */
    SocketKeepAlive.prototype.reset = function () {
        this.stop();
        this.keepAlive = setTimeout(this.boundCheckConnection, 20000);
    };
    /**
     *
     */
    SocketKeepAlive.prototype.stop = function () {
        if (this.keepAlive !== null) {
            clearTimeout(this.keepAlive);
            this.keepAlive = null;
        }
    };
    /**
     * TODO: Move variable to config/constant file.
     */
    SocketKeepAlive.prototype.check = function () {
        this.lastPing = Date.now();
        this.sendPing();
        this.connectionTimer = setTimeout(this.boundTimedOut, 5000);
    };
    /**
     * TODO: Do something with the latency data. Prob something with events and sending it out.
     */
    SocketKeepAlive.prototype.finish = function () {
        this.latency = Date.now() - this.lastPing;
        this.clear();
        this.reset();
    };
    /**
     *
     */
    SocketKeepAlive.prototype.clear = function () {
        clearTimeout(this.connectionTimer);
        this.lastPing = null;
        this.connectionTimer = null;
    };
    /**
     * TODO: Do something when socket is timed out. Prob closing the connection.
     */
    SocketKeepAlive.prototype.timedOut = function () {
    };
    /**
     * TODO: Move creation of this object to constant and to WebSocketFrame class.
     */
    SocketKeepAlive.prototype.sendPing = function () {
        var frame = new tempWebSocketFrame(Buffer.alloc(4), Buffer.alloc(10), { maxReceivedFrameSize: 0x10000 });
        frame.opcode = 0x09;
        frame.fin = true;
        this.socket.write(frame.toBuffer(), 'binary');
    };
    /**
     * Destroys this instance of SocketKeepAlive.
     */
    SocketKeepAlive.prototype.destroyInstance = function () {
        this.clear();
        this.stop();
        this.socket = null;
        this.keepAlive = null;
        this.connectionTimer = null;
        this.lastPing = null;
        this.latency = null;
        this.boundCheckConnection = null;
        this.boundTimedOut = null;
    };
    return SocketKeepAlive;
}());
exports.SocketKeepAlive = SocketKeepAlive;
