import { Server } from '../base/Server';
import { Client } from '../base/Client';
import { Message } from '../base/Message';
import { GameObject } from './GameObject';
/**
 *
 */
export declare const enum GameServerEvents {
    UPDATE = "update"
}
/**
 * @author Bram Willem van der Kuip <vdkuipb@gmail.com>
 */
export declare class GameServer extends Server {
    /**
     *
     * @type {GameObject[]}
     */
    protected gameObjects: GameObject[];
    /**
     * Creates an instance of Server.
     * @param {string} [ip='127.0.0.1']
     * @param {number} [port=3000]
     */
    constructor(ip?: string, port?: number);
    /**
     *
     */
    start(): void;
    /**
     *
     * @param {Client} client
     */
    protected onClientConnection(client: Client): void;
    /**
     *
     * @param {Client} client
     * @param {Message} message
     */
    protected onClientData(client: Client, message: Message): void;
    /**
     *
     * @param {Client} client
     */
    protected onClientClose(client: Client): void;
    /**
     *
     * @param {GameObject} gameObject
     */
    instantiate(gameObject: GameObject): void;
    /**
     *
     * @param {(GameObject | string)} gameObject
     */
    destroy(gameObject: GameObject): void;
    destroy(id: string): void;
    /**
     *
     * @param {number} [delta]
     */
    update(delta?: number): void;
    /**
     *
     * @param {string} id
     * @returns {GameObject}
     */
    protected getGameObjectById(id: string): GameObject;
    /**
     *
     */
    destroyInstance(): void;
}
