"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var Server_1 = require("../base/Server");
/**
 *
 *
 * @param {object[]} players
 * @returns {Message}
 */
function updateMessage(gameObjects) {
    return {
        event: "update" /* UPDATE */,
        data: gameObjects,
    };
}
/**
 * @author Bram Willem van der Kuip <vdkuipb@gmail.com>
 */
var GameServer = /** @class */ (function (_super) {
    __extends(GameServer, _super);
    /**
     * Creates an instance of Server.
     * @param {string} [ip='127.0.0.1']
     * @param {number} [port=3000]
     */
    function GameServer(ip, port) {
        if (ip === void 0) { ip = '127.0.0.1'; }
        if (port === void 0) { port = 3000; }
        return _super.call(this, ip, port) || this;
    }
    /**
     *
     */
    GameServer.prototype.start = function () {
        this.gameObjects = [];
        _super.prototype.start.call(this);
    };
    /**
     *
     * @param {Client} client
     */
    GameServer.prototype.onClientConnection = function (client) {
    };
    /**
     *
     * @param {Client} client
     * @param {Message} message
     */
    GameServer.prototype.onClientData = function (client, message) {
    };
    /**
     *
     * @param {Client} client
     */
    GameServer.prototype.onClientClose = function (client) {
    };
    /**
     *
     * @param {GameObject} gameObject
     */
    GameServer.prototype.instantiate = function (gameObject) {
        this.gameObjects.push(gameObject);
    };
    GameServer.prototype.destroy = function (gameObject) {
        var id = typeof (gameObject) === 'string' ? gameObject : gameObject.id;
        for (var i = this.gameObjects.length - 1; i >= 0; i -= 1) {
            if (this.gameObjects[i].id === id) {
                this.gameObjects[i].destroyInstance();
                this.gameObjects.splice(i, 1);
            }
        }
    };
    /**
     *
     * @param {number} [delta]
     */
    GameServer.prototype.update = function (delta) {
        for (var i = this.gameObjects.length - 1; i >= 0; i -= 1) {
            this.gameObjects[i].update(0);
        }
        this.broadcast(updateMessage(this.gameObjects.map(function (go) {
            var data = go.toData();
            data.name = go.constructor.name;
            data.id = go.id;
            return data;
        })));
    };
    /**
     *
     * @param {string} id
     * @returns {GameObject}
     */
    GameServer.prototype.getGameObjectById = function (id) {
        for (var i = this.gameObjects.length - 1; i >= 0; i -= 1) {
            if (this.gameObjects[i].id === id) {
                return this.gameObjects[i];
            }
        }
    };
    /**
     *
     */
    GameServer.prototype.destroyInstance = function () {
        _super.prototype.destroyInstance.call(this);
    };
    return GameServer;
}(Server_1.Server));
exports.GameServer = GameServer;
