import { WebSocketFrame } from '../utils/WebSocketFrame';
import { BaseMessageEvents } from './Server';

const typeString: string = 'string';
const typeNumber: string = 'number';
const typeObject: string = 'object';

/**
 * @author Bram Willem van der Kuip <vdkuipb@gmail.com>
 */
export interface Message {
    event: number | string;
    data: any;
}
/**
 *
 */
const closeMessage: Message = {
    event: BaseMessageEvents.CLOSE,
    data: {},
};
/**
 *
 */
const pongMessage: Message = {
    event: BaseMessageEvents.PONG,
    data: {},
};
/**
 *
 */
const INVALID_STRING_PARSE = 'invalid_string_parse';
/**
 *
 */
const INVALID_MESSAGE_OBJECT = 'invalid_message_object';
/**
 *
 */
function invalidDataMessage(type: string): Message {
    return {
        event: BaseMessageEvents.INVALID_MESSAGE,
        data: { type },
    };
}
/**
 *
 */
const opcodePong: number = 0x0A;
/**
 *
 */
const opcodeClose: number = 0x08;

/**
 * TODO: rename.
 *
 * @author Bram Willem van der Kuip <vdkuipb@gmail.com>
 */
export class MessageFormatter {

    /**
     *
     * @param {Message} message
     * @returns {string}
     */
    public static encrypt(message: Message): string {
        return WebSocketFrame.to(message).toBuffer();
    }

    /**
     *
     * @param {object} data
     * @return {Message}
     */
    public static decrypt(data: object): Message {
        const frame: any = WebSocketFrame.from(data);

        if (frame.opcode === opcodePong) {
            return pongMessage;
        }

        if (frame.opcode === opcodeClose) {
            return closeMessage;
        }

        if (!frame.binaryPayload || frame.binaryPayload.length === 0) {
            return;
        }

        const dataString: string = frame.binaryPayload.toString();
        let dataObject;

        try {
            dataObject = JSON.parse(dataString);
        } catch (error) {
            return invalidDataMessage(INVALID_STRING_PARSE);
        }

        if (!this.isMessage(dataObject)) {
            return invalidDataMessage(INVALID_MESSAGE_OBJECT);
        }

        return dataObject;
    }

    /**
     * Checks if object implements the Message interface.
     * @param {*} data The object the check if is a message.
     * @returns {boolean} The flag if it is true of not.
     */
    public static isMessage(data: any): boolean {
        return data &&
            data.event &&
            (typeof(data.event) === typeString ||
            typeof(data.event) === typeNumber) &&
            data.data !== undefined
    }

}
