import { Socket } from 'net';
import { Destroyable } from '../utils/Destroyable';

const tempWebSocketFrame = require('../vendor/websocket/lib/WebSocketFrame.js');

/**
 * @author Bram Willem van der Kuip <vdkuipb@gmail.com>
 */
export class SocketKeepAlive implements Destroyable {
    /**
     *
     */
    private socket: Socket;
    /**
     *
     */
    private keepAlive: NodeJS.Timeout;
    /**
     *
     */
    private connectionTimer: NodeJS.Timeout;
    /**
     *
     */
    private latency: number;
    /**
     *
     */
    private lastPing: number;
    /**
     *
     */
    private boundCheckConnection: () => any;
    private boundTimedOut: () => any;
    /**
     * Creates an instance of SocketKeepAlive.
     */
    public constructor(socket: Socket) {
        this.socket = socket;
        this.keepAlive = null;
        this.latency = 0;
        this.lastPing = 0;

        this.boundCheckConnection = this.check.bind(this);
        this.boundTimedOut = this.timedOut.bind(this);

        this.reset();
    }

    /**
     * TODO: Move variable to config/constant file.
     */
    private reset(): void {
        this.stop();
        this.keepAlive = setTimeout(this.boundCheckConnection, 20000);
    }

    /**
     *
     */
    private stop(): void {
        if (this.keepAlive !== null) {
            clearTimeout(this.keepAlive);
            this.keepAlive = null;
        }
    }

    /**
     * TODO: Move variable to config/constant file.
     */
    private check(): void {
        this.lastPing = Date.now();
        this.sendPing();
        this.connectionTimer = setTimeout(this.boundTimedOut, 5000);
    }

    /**
     * TODO: Do something with the latency data. Prob something with events and sending it out.
     */
    public finish(): void {
        this.latency = Date.now() - this.lastPing;
        this.clear();
        this.reset();
    }

    /**
     *
     */
    private clear(): void {
        clearTimeout(this.connectionTimer);
        this.lastPing = null;
        this.connectionTimer = null;
    }

    /**
     * TODO: Do something when socket is timed out. Prob closing the connection.
     */
    private timedOut(): void {

    }

    /**
     * TODO: Move creation of this object to constant and to WebSocketFrame class.
     */
    public sendPing(): void {
        const frame = new tempWebSocketFrame(
            Buffer.alloc(4),
            Buffer.alloc(10),
            { maxReceivedFrameSize: 0x10000 },
        );
        frame.opcode = 0x09;
        frame.fin = true;
        this.socket.write(frame.toBuffer(), 'binary');
    }

    /**
     * Destroys this instance of SocketKeepAlive.
     */
    public destroyInstance(): void {
        this.clear();
        this.stop();

        this.socket = null;
        this.keepAlive = null;
        this.connectionTimer = null;
        this.lastPing = null;
        this.latency = null;
        this.boundCheckConnection = null;
        this.boundTimedOut = null;
    }
}
