import { Server } from '../base/Server';
import { Client } from '../base/Client';
import { Message } from '../base/Message';
import { GameObject } from './GameObject';

/**
 *
 */
export const enum GameServerEvents {
    UPDATE = 'update',
}
/**
 *
 *
 * @param {object[]} players
 * @returns {Message}
 */
function updateMessage(gameObjects: object[]): Message {
    return {
        event: GameServerEvents.UPDATE,
        data: gameObjects,
    };
}

/**
 * @author Bram Willem van der Kuip <vdkuipb@gmail.com>
 */
export class GameServer extends Server {
    /**
     *
     * @type {GameObject[]}
     */
    protected gameObjects: GameObject[];
    /**
     * Creates an instance of Server.
     * @param {string} [ip='127.0.0.1']
     * @param {number} [port=3000]
     */
    public constructor(ip: string = '127.0.0.1', port: number = 3000) {
        super(ip, port);
    }

    /**
     *
     */
    public start(): void {
        this.gameObjects = [];

        super.start();
    }

    /**
     *
     * @param {Client} client
     */
    protected onClientConnection(client: Client): void {

    }

    /**
     *
     * @param {Client} client
     * @param {Message} message
     */
    protected onClientData(client: Client, message: Message): void {

    }

    /**
     *
     * @param {Client} client
     */
    protected onClientClose(client: Client): void {

    }

    /**
     *
     * @param {GameObject} gameObject
     */
    public instantiate(gameObject: GameObject): void {
        this.gameObjects.push(gameObject);
    }

    /**
     *
     * @param {(GameObject | string)} gameObject
     */
    public destroy(gameObject: GameObject): void;
    public destroy(id: string): void;
    public destroy(gameObject: GameObject | string): void {
        const id = typeof(gameObject) === 'string' ? gameObject : gameObject.id;
        for (let i = this.gameObjects.length - 1; i >= 0; i -= 1) {
            if (this.gameObjects[i].id === id) {
                this.gameObjects[i].destroyInstance();
                this.gameObjects.splice(i, 1);
            }
        }
    }

    /**
     *
     * @param {number} [delta]
     */
    public update(delta?: number): void {
        for (let i = this.gameObjects.length - 1; i >= 0; i -= 1) {
            this.gameObjects[i].update(0);
        }

        this.broadcast(updateMessage(this.gameObjects.map((go: GameObject) => {
            const data: any = go.toData();
            data.name = go.constructor.name;
            data.id = go.id;
            return data;
        })));
    }

    /**
     *
     * @param {string} id
     * @returns {GameObject}
     */
    protected getGameObjectById(id: string): GameObject {
        for (let i = this.gameObjects.length - 1; i >= 0; i -= 1) {
            if (this.gameObjects[i].id === id) {
                return this.gameObjects[i];
            }
        }
    }

    /**
     *
     */
    public destroyInstance(): void {

        super.destroyInstance();
    }

}
